import enum
import itertools
import pathlib


class NounTypes(enum.Enum):
    """The various types of nouns supported by memorable."""

    ALCOHOLS = 'alcohols'
    ANIMALS = 'animals'
    ELEMENTS = 'elements'
    FOODS = 'foods'
    FRUITS = 'fruits'
    GEOGRAPHIES = 'geographies'
    HOUSEHOLDS = 'households'
    INVESTMENTS = 'investments'
    LITERATURE = 'literature'
    MYTHICAL = 'mythical'
    OCCUPATIONS = 'occupations'
    ORGANS = 'organs'
    PLACES = 'places'
    RELATIONS = 'relations'
    ROCKS = 'rocks'
    ROYALTY = 'royalty'
    TOOLS = 'tools'
    TRANSPORTS = 'transports'
    TREES = 'trees'
    VEGTABLES = 'vegtables'
    WATER_BODIES = 'water_bodies'


_PROJECT_DIR = pathlib.Path(__file__).parent
_RESOURCES_DIR = _PROJECT_DIR.joinpath('_resources')


def _load_resource(resource: str) -> list:
    """Load the given resource file"""
    return _RESOURCES_DIR.joinpath(
        f'{resource}.txt'
    ).read_text().strip().split('\n')


NAMES = _load_resource('names')
ADJECTIVES = _load_resource('adjectives')
ADVERBS = _load_resource('adverbs')
VERBS = _load_resource('verbs')

NOUN_MAP = {
    noun: _load_resource(f'nouns/{noun.value}') for noun in NounTypes
}

NOUNS = list(itertools.chain.from_iterable(NOUN_MAP.values()))



import secrets
import string

from memorable import _word_bank

_EXTRA_CHARS = string.ascii_lowercase + string.digits

NON_NAMES = list(set(
    _word_bank.NOUNS
    + _word_bank.VERBS
    + _word_bank.ADJECTIVES
    + _word_bank.ADVERBS
))

def name(extra_characters: int = 0) -> str:
    """
    Create a memorable name.

    :param extra_characters:
        The number of extra chracters that should be added to the
        end of the string. This is intended to help reduce collisions if
        the string is being used as an ID.
    """
    name = secrets.choice(_word_bank.NAMES)
    adjective = secrets.choice(_word_bank.ADJECTIVES)
    adverb = secrets.choice(_word_bank.ADVERBS)
    result = f'{name}-the-{adverb}-{adjective}'
    return _pad_with_extra_characters(result, extra_characters)


def action(extra_characters: int = 0) -> str:
    """
    Create a memorable action.

    :param extra_characters:
        The number of extra chracters that should be added to the
        end of the string. This is intended to help reduce collisions if
        the string is being used as an ID.
    """
    verb = secrets.choice(_word_bank.VERBS)
    adjective = secrets.choice(_word_bank.ADJECTIVES)
    noun = secrets.choice(_word_bank.NOUNS)
    article = secrets.choice(['a', 'the'])
    article = 'an' if adjective[0] in 'aeiou' and article == 'a' else article
    result = f'{verb}-{article}-{adjective}-{noun}'
    return _pad_with_extra_characters(result, extra_characters)


def thing(kind: _word_bank.NounTypes = None, extra_characters: int = 0) -> str:
    """
    Create a memorable thing.

    :param kind:
        The kind of thing to create a string for. This can be an animal,
        element, occupation, etc.
    :param extra_characters:
        The number of extra chracters that should be added to the
        end of the string. This is intended to help avoid collisions.
    """
    adjective = secrets.choice(_word_bank.ADJECTIVES)
    adverb = secrets.choice(_word_bank.ADVERBS)
    noun_population = (
        _word_bank.NOUNS if kind is None else _word_bank.NOUN_MAP[kind]
    )
    noun = secrets.choice(noun_population)
    result = f'{adverb}-{adjective}-{noun}'
    return _pad_with_extra_characters(result, extra_characters)


def code_phrase(words: int = 4) -> str:
    """
    Create a random phrase of four words. No effort is taken to make
    it appear as a coherent english phrase.

    :param words:
        The number of words to include in the phrase.
    """
    return '-'.join([secrets.choice(NON_NAMES) for _ in range(words)])


def _pad_with_extra_characters(base: str, count: int) -> str:
    """
    Pad the given base string out with `count` random characters.

    :param base:
        The base string value to append to.
    :param count:
        The number of characters to add.
    :return:
        The base padded with the specified number of extra characters.
    """
    if count == 0:
        return base
    padding = ''.join(secrets.choice(_EXTRA_CHARS) for _ in range(count))
    return f'{base}-{padding}'


from memorable._poet import action
from memorable._poet import code_phrase
from memorable._poet import name
from memorable._poet import thing

from memorable._word_bank import NounTypes

from memorable._word_bank import ADJECTIVES
from memorable._word_bank import ADVERBS
from memorable._word_bank import NOUNS
from memorable._word_bank import VERBS
from memorable._word_bank import NAMES

__all__ = (
    'action',
    'code_phrase',
    'name',
    'thing',
    'NounTypes',
    'ADJECTIVES',
    'ADVERBS',
    'NOUNS',
    'VERBS',
    'NAMES'
)


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['memorable']

package_data = \
{'': ['*'], 'memorable': ['_resources/*', '_resources/nouns/*']}

setup_kwargs = {
    'name': 'memorable',
    'version': '1.0.2',
    'description': 'A library for creating memorable strings.',
    'long_description': "# Memorable\n\nA library for generating memorable strings.\n\n## Usage\n\n```python\nimport memorable\n\nmemorable.name()\n# lue-the-jealously-happy-go-lucky\nmemorable.action()\n# eat-a-pretty-payment\nmemorable.thing()\n# cleverly-neglected-talk\nmemorable.code_phrase()\n# amused-chilly-minestrone-distribute\n```\n\n## Advanced usage\n\nIt is possible to generate strings focusing on a specific theme.\n\n```python\nfor kind in memorable.NounTypes:\n    phrase = memorable.thing(kind=kind)\n    print(f'{kind.value}: {phrase}')\n    # alcohols: wonderfully-overdue-beer\n    # animals: uncritically-thoughtful-octopus\n    # elements: vividly-mindless-chromium\n    # foods: jubilantly-snappy-stew\n    # fruits: poorly-handmade-kiwano\n    # geographies: quaintly-mountainous-beach\n    # households: hurriedly-ready-zester\n    # investments: meticulously-variable-bitcoin\n    # literature: madly-bogus-ode\n    # mythical: readily-general-centaur\n    # occupations: casually-tinted-data-scientist\n    # organs: nightly-splendid-lungs\n    # places: casually-definitive-farm\n    # relations: later-ready-babushka\n    # rocks: slavishly-hot-picrite\n    # royalty: fast-loyal-despot\n    # tools: unnecessarily-unripe-needle-nose\n    # transports: hollowly-cheerful-pickup\n    # trees: never-offbeat-fir\n    # vegtables: naively-happy-bitter-melon\n    # water_bodies: upliftingly-flickering-wadi\n```\n\nIf `memorable` is being used to generate ids and you'd like to reduce the\nfrequency of collisions (you should expect some collisions) it's possible\nto request some extra characters tacked on the end to make them less common.\n\n```python\nmemorable.action(extra_characters=4)\n# demand-the-hidden-bread-machine-gths\n```",
    'author': 'Kevin Schiroo',
    'author_email': 'kjschiroo@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://gitlab.com/kjschiroo/memorable',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
